#!/bin/bash
if [ -z "$VARID_FOLDER" ]; then
  VARID_FOLDER="$(cd "$(dirname $0)/.."; pwd)"
  echo 1>&2 "warning: VARID_FOLDER undefined; assuming ${VARID_FOLDER}"
fi

if [ ! -d "$VARID_FOLDER/helper/" ]; then
  echo 1>&2 "error: cannot find directory $VARID_FOLDER/helper/"
  exit 1
fi

SPLIT_SIZE=10000
OVERLAP_SIZE=100
THREADS=1

usage () { 
	echo $0 [ --alignments filename ] [ --ref-file filename ] [ --donor-name name ] [ --output filename ] [ --temp-dir path ] [ --split-size 10000 \{default\} ] [ --overlap-size \{default 100\} ] [ --script ] [ --threads 1 ]
}

if [ $# -lt 5 ]
then
	usage
	exit 1
fi

SCRIPT=0
until [ -z "$1" ]; do
  case $1 in
    "--alignments")
      shift
      ALIGNMENTS_FILE=$1; shift;;
    "--ref-file")
      shift
      REFERENCE_FILE=$1; shift;;
    "--output")
      shift
      OUTPUT_FILE=$1; shift;;
    "--overlap-size")
      shift
      OVERLAP_SIZE=$1; shift;;
    "--split-size")
      shift
      SPLIT_SIZE=$1; shift;;
    "--threads")
      shift
      THREADS=$1; shift;;
    "--donor-name")
      shift
      DONOR_NAME=$1; shift;;
    "--temp-dir")
      shift
      TEMP_DIR=$1; shift;;
    "--script")
      SCRIPT=1
      shift;;
    *) 
	usage
	echo Option $1 does not exist!
	exit 1
	break;;
  esac
done

if [ -z "${ALIGNMENTS_FILE}" -o -z "${REFERENCE_FILE}" -o -z "${OUTPUT_FILE}" -o -z "${TEMP_DIR}" ]; then 
	usage
	exit 1
fi

mkdir -p ${TEMP_DIR}

rm ${OUTPUT_FILE}

#get the reference file length
fasta_entries=`grep ">" ${REFERENCE_FILE} | wc -l`
if [ ${fasta_entries} -gt 1 ]; then
	echo "VARiD helper script does not currenty support, please provide only a single input contig at a time"
	echo "The alignment SAM file may contain alignments to other contigs"
	exit
fi
lines=`grep -v ">" ${REFERENCE_FILE} | wc -l`
characters=`grep -v ">" ${REFERENCE_FILE} | wc -c`
reference_size=`expr $characters - $lines`
echo Detected reference\(s\) size is ${reference_size} base pairs

i=1
while [ $i -le ${reference_size} ]
do
	if [ $SCRIPT -eq 0 ]; then
		echo running "${VARID_FOLDER}/varid_exec -a ${ALIGNMENTS_FILE} -r ${REFERENCE_FILE} -n ${DONOR_NAME} -d ${TEMP_DIR} --predict-start $i --predict-end $(expr $(expr ${SPLIT_SIZE} + $i ) - 1) --hmm-area ${OVERLAP_SIZE} --threads $THREADS >> ${OUTPUT_FILE}"
		${VARID_FOLDER}/varid_exec -a ${ALIGNMENTS_FILE} -r ${REFERENCE_FILE} -n ${DONOR_NAME} -d ${TEMP_DIR} --predict-start $i --predict-end $(expr $(expr ${SPLIT_SIZE} + $i ) - 1) --hmm-area ${OVERLAP_SIZE} --threads $THREADS >> ${OUTPUT_FILE}
	else
		echo ${VARID_FOLDER}/varid_exec -a ${ALIGNMENTS_FILE} -r ${REFERENCE_FILE} -n ${DONOR_NAME} -d ${TEMP_DIR} --predict-start $i --predict-end $(expr $(expr ${SPLIT_SIZE} + $i ) - 1) --hmm-area ${OVERLAP_SIZE} --threads $THREADS \>\> ${OUTPUT_FILE}
	fi
	i=`expr $i + ${SPLIT_SIZE}`
done

