/*
 * VARiD: A Variation Detection Framework for Colorspace and Letterspace platforms
 * Contact: <varid at cs dot toronto dot edu>
 * Copyright 2010 the authors (see README).
 *
 * This file is part of VARiD.
 *
 * VARiD is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * VARiD is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with VARiD.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <unistd.h>
#include <stdio.h>
#include <string.h>
#include <ctype.h>
#include <sys/stat.h>
#include "va_assert.h"
#include "va_log.h"
#include "varid.h"


extern double*  cs_error_cnt;
extern double*  cs_error_dpts;
extern unsigned int cs_error_size;
extern double*  ls_error_cnt;
extern double*  ls_error_dpts;
extern unsigned int ls_error_size;



CData_t real_cdata_prep(
    int       isDip) {
    CData_t cdata;
    memset(&cdata, 0, sizeof(cdata));

    cdata.src = SRC_REAL;

    /* diploid ? */
    cdata.mode = (isDip) ? MODE_DIPLOID : MODE_HAPLOID;
    
    /* stuff that should be filled in */
    cdata.ref.size = -1;

    int i;
    if (load_learned_file==NULL) {
	    for (i = 0; i < cs_error_size; i++) {
		if (cs_error_dpts[i] > err_coverage) {
			cdata.error_mat_cs[i] = MAX(cs_error_cnt[i]/cs_error_dpts[i],min_cs_err);
			cdata.error_mat_cs[i] = MIN(cdata.error_mat_cs[i], max_cs_err);
		} else {
			cdata.error_mat_cs[i] = cs_err;   
		}
	    }
	    for (i = 0; i < ls_error_size; i++) {
		if (ls_error_dpts[i] > err_coverage) {
			cdata.error_mat_ls[i] = MAX(ls_error_cnt[i]/ls_error_dpts[i],min_ls_err);
			cdata.error_mat_ls[i] = MIN(cdata.error_mat_ls[i], max_ls_err);
		} else {
			cdata.error_mat_ls[i] = ls_err;
		}
	    }
    } else {
	    char buffer[BUFSIZ]; buffer[0]='\0';
	    int j, res; double err,cnt,dpts;
	    while(fgets(buffer,BUFSIZ,load_learned_file) && buffer[0]!='#');
	    for (i=0; i<MAX_QVAL; i++ ) {
		res = fscanf(load_learned_file,"%d: %lf %lf %lf\n",&j,&err,&cnt,&dpts);
		if (res!=4) {
			fprintf(stderr,"There has been a fatal error in loading letter space read error values\n");
			fclose(load_learned_file);
		}
		VA_ASSERT(j==i);
		cdata.error_mat_ls[i]=err;
		ls_error_cnt[i]=cnt;
		ls_error_dpts[i]=dpts;
	    }
	    while(fgets(buffer,BUFSIZ,load_learned_file) && buffer[0]!='#');
	    for (i=0; i<MAX_QVAL; i++ ) {
		res = fscanf(load_learned_file,"%d: %lf %lf %lf\n",&j,&err,&cnt,&dpts);
		if (res!=4) {
			fprintf(stderr,"There has been a fatal error in loading colour space read error values\n");
			fclose(load_learned_file);
		}
		VA_ASSERT(j==i);
		cdata.error_mat_cs[i]=err;
		cs_error_cnt[i]=cnt;
		cs_error_dpts[i]=dpts;
	    }
	    fclose(load_learned_file);
    }

    if (save_learned_file!=NULL) {
	    fprintf(save_learned_file,"#Letter space read errors:\n");
	    for (i=0; i<ls_error_size; i++) {
		fprintf(save_learned_file,"%4d: %12.8e\t%f\t%f\n",i,cdata.error_mat_ls[i],ls_error_cnt[i],ls_error_dpts[i]);
	    }
	    fprintf(save_learned_file,"#Colour space read errors:\n");
	    for (i=0; i<cs_error_size; i++) {
		fprintf(save_learned_file,"%4d: %12.8e\t%f\t%f\n",i,cdata.error_mat_cs[i],cs_error_cnt[i],cs_error_dpts[i]);
    	    }
	    exit(1);
    }

    cdata.ref.data = NULL;
    return (cdata);
}
